/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
* File Name    : pwm.c
* Version      : Applilet EZ PL for RL78
* Device(s)    : R5F10Y14,R5F10Y16,R5F10Y17,R5F10Y44,R5F10Y46,R5F10Y47
* Tool-Chain   : CC-RL
* Description  : This file implements main function.
* Creation Date: 2016/09/14
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "sfr.h"
#include "common.h"
#include "config.h"
#include "pwm.h"

#if defined PWM_USED
/*******************************************************************************
Macro definitions
*******************************************************************************/
/*  PORT Assignment		*/
#ifdef __CCRL__
#define	PORT_PWM_CH1	P0_bit.no4
#define	PORT_PWM_CH2	P0_bit.no5
#define	PORT_PWM_CH3	P0_bit.no7

#else
#error "undefined compiler"
#endif

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
/*  Structure Definition	*/
#ifdef __CCRL__
__saddr PWM_TBL tblgPWM[PWM_USED];
__saddr UCHAR	ucgPWM_Cnt = 0;
__saddr UCHAR	ucLoopCount;
__saddr ULONG	ulFprs;
__saddr ULONG	ulDivideWork;
__saddr ULONG	ulRoundWork;
__saddr ULONG	ulTDR00;

#else
#error "undefined compiler"
#endif

/*	Function Prototype	*/
void PWM_CH0_Init( ULONG Freq );
void PWM_CH1_Init( ULONG Freq );
void PWM_CH2_Init( ULONG Freq );
void PWM_CH3_Init( ULONG Freq );
void PWM_Out_CH1( void );
void PWM_Out_CH2( void );
void PWM_Out_CH3( void );
UCHAR PWM_Calc_Duty( UCHAR regTDR00L );

/*******************************************************************************
Private variables and functions
*******************************************************************************/

/*******************************************************************************
* Function Name: PWM_CH1_Init
* Description  : initializing the PWM output that uses the Timer Array Unit(Cannel1).
* Arguments    : Freq    ... setting frequency
* Return Value : none
********************************************************************************/
void PWM_CH1_Init( ULONG Freq )
{
#if ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH1_USED) )
	/* Enables timer array unit input clock supply(TAU0EN=1) */
	PER0 |= 0x01;

	TPS0  = 0x00;
	TTH0 |= 0x02;			/* TTH01 = 1 */
	TT0  |= 0x03;			/* TT00 = 1, TT01 = 1 */
	
	PWM_CH0_Init(Freq);

	TMR01H = 0x04;
	TMR01L = 0x09;
	TOM0 |= 0x02;			/* TOM01 = 1 */
	TOL0 &= (UCHAR)~0x02;	/* TOL01 = 0 */
	TO0  &= (UCHAR)~0x02;	/* TO01  = 0 */
	TOE0 &= (UCHAR)~0x02;	/* TOE01 = 0 */

	IF0L &= (UCHAR)~0x80;	/* TMIF00 = 0 */
	MK0L &= (UCHAR)~0x80;	/* TMMK00 = 0 */
	TS0  |= 0x01;			/* TS00   = 1(timer start) */
#endif /* end of ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH1_USED) ) */
}

/*******************************************************************************
* Function Name: PWM_Param
* Description  : duty setting.
* Arguments    : Duty ... duty ratio
*                In   ... enable/disable
* Return Value : none
********************************************************************************/
void PWM_Param( UCHAR Duty, UCHAR In )
{
	if (In == 1){
		if (tblgPWM[ucgPWM_Cnt].Input == LOW){
			tblgPWM[ucgPWM_Cnt].SetDuty = Duty;
			tblgPWM[ucgPWM_Cnt].Input = 1;

		}else if (tblgPWM[ucgPWM_Cnt].Input == HIGH){
			tblgPWM[ucgPWM_Cnt].SetDuty = 0;
		}
	}
}

/*******************************************************************************
* Function Name: PWM_Out_CH1
* Description  : PWM that uses the Timer Array Unit(Cannel1) output.
* Arguments    : none
* Return Value : none
********************************************************************************/
void PWM_Out_CH1( void )
{
#if ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH1_USED) )
	UCHAR	ucreg;
	UCHAR	regTDR01L;

	if (tblgPWM[ucgPWM_Cnt].Input == HIGH){
		if (tblgPWM[ucgPWM_Cnt].SetDuty != tblgPWM[ucgPWM_Cnt].OutDuty){
			switch (tblgPWM[ucgPWM_Cnt].SetDuty){
				case 0:
					TT0  |= 0x02;			/* TT01  = 1(timer stop) */
					TOE0 &= (UCHAR)~0x02;	/* TOE01 = 0 */
					PORT_PWM_CH1 = LOW;
					break;
				case 100:
					TT0  |= 0x02;			/* TT01  = 1(timer stop) */
					TOE0 &= (UCHAR)~0x02;	/* TOE01 = 0 */
					PORT_PWM_CH1 = HIGH;
					break;
				default:
					ucreg  = TDR00L;
					regTDR01L = PWM_Calc_Duty(ucreg);
					TT0  |= 0x02;			/* TT01  = 1(timer stop) */
					TDR01H = 0x00;
					TDR01L = regTDR01L;
					PORT_PWM_CH1 = LOW;
					TOE0 |= 0x02;			/* TOE01 = 1 */
					TS0  |= 0x02;			/* TS01  = 1(timer start) */
					break;
			}
		}

	}else if (tblgPWM[ucgPWM_Cnt].Input == LOW){
		tblgPWM[ucgPWM_Cnt].SetDuty = 0;
		TT0  |= 0x02;						/* TT01  = 1(timer stop) */
		TOE0 &= (UCHAR)~0x02;				/* TOE01 = 0 */
		PORT_PWM_CH1 = LOW;
	}
	tblgPWM[ucgPWM_Cnt].OutDuty = tblgPWM[ucgPWM_Cnt].SetDuty;
#endif /* end of ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH1_USED) ) */
}

/*******************************************************************************
* Function Name: PWM_Calc_Duty
* Description  : The register setting value corresponding to the duty is calculated.
* Arguments    : regTDR00L ... TDR0nL setting value
*                Duty    ... duty ratio
* Return Value : CMP1n setting value
********************************************************************************/
UCHAR PWM_Calc_Duty( UCHAR regTDR00L )
{
	ULONG	ulwork1 = 0;
	UCHAR	ucwork2 = 0;
	UCHAR	ucwork3 = 0;
	
	ulwork1 = regTDR00L;
	ulwork1 = (ulwork1 + 1) * tblgPWM[ucgPWM_Cnt].SetDuty;
	ucwork2 = (UCHAR)(ulwork1 % 100);
	ucwork3 = (UCHAR)(ulwork1 / 100);
	
	if (ucwork2 >= 50)	ucwork3 ++;
	
	if (ucwork3 == 0)	ucwork3 ++;
	
	return ucwork3;
}

/*******************************************************************************
* Function Name: PWM_Counter_Clear
* Description  : PWM Panel Counter Clear.
* Arguments    : none
* Return Value : none
********************************************************************************/
void PWM_Counter_Clear( void )
{
	ucgPWM_Cnt = 0;
}

/*******************************************************************************
* Function Name: PWM_Initialize
* Description  : PWM Initialize Process.
* Arguments    : OutPort
*                Freq
* Return Value : none
********************************************************************************/
void PWM_Initialize( UCHAR OutPort, ULONG Freq )
{
	tblgPWM[ucgPWM_Cnt].Input = 0;
	tblgPWM[ucgPWM_Cnt].SetDuty = 0;
	tblgPWM[ucgPWM_Cnt].OutDuty = 0;
	tblgPWM[ucgPWM_Cnt].Port = OutPort;

	switch (OutPort){		/* P04,P05,P07 comes without fail. */
		case 4:
			PWM_CH1_Init(Freq);
			break;
		case 5:
			PWM_CH2_Init(Freq);
			break;
		case 7:
			PWM_CH3_Init(Freq);
			break;
	}

	ucgPWM_Cnt++;
}

/*******************************************************************************
* Function Name: PWM_Out_Proc
* Description  : PWM Out Processing.
* Arguments    : none
* Return Value : none
********************************************************************************/
void PWM_Out_Proc( void )
{
	switch (tblgPWM[ucgPWM_Cnt].Port){
		case 4:
			PWM_Out_CH1();
			break;
		case 5:
			PWM_Out_CH2();
			break;
		case 7:
			PWM_Out_CH3();
			break;
	}

	tblgPWM[ucgPWM_Cnt].Input = LOW;
	ucgPWM_Cnt++;
}

/*******************************************************************************
* Function Name: PWM_CH0_Init
* Description  : initializing the PWM output that uses the Timer Array Unit(Cannel0).
* Arguments    : Freq    ... setting frequency
* Return Value : none
********************************************************************************/
void PWM_CH0_Init( ULONG Freq )
{
	USHORT	ushDivide[16] = {1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768};
	UCHAR	ucsysclocksrc;
	UCHAR	ucsysclockfreq;
	
	/* Enables timer array unit input clock supply(TAU0EN=1) */
	PER0 |= 0x01;

	TT0  |= 0x01;			/* TT00 = 1, TT01 = 1 */
	
	TMR00H = 0x00;
	TMR00L = 0x01;
	
	ucsysclocksrc  = SYS_CLOCK_SRC;
	ucsysclockfreq = SYS_CLOCK_FREQ;
	ulFprs = SYS_CLOCK_FREQ * 1000000;
	if( ucsysclocksrc == CLOCK_SRC_INTERNAL){
		if(ucsysclockfreq == 1){
			ulFprs = 1250000;
		}else if(ucsysclockfreq == 2){
			ulFprs = 2500000;
		}
	}
	ulDivideWork = ulFprs / (Freq * 256);
	ulRoundWork = ulFprs % (Freq * 256);
	if(ulRoundWork > 0) {
		ulDivideWork += 1;
	}
	for(ucLoopCount = 0; ucLoopCount < 16; ucLoopCount++) {
		if(ushDivide[ucLoopCount] >= (short)(ulDivideWork)) {
			ulDivideWork = ushDivide[ucLoopCount];
			TPS0 = ucLoopCount & 0x0F;
			ulTDR00 = ulFprs / (ulDivideWork * Freq) - 1;
			ulRoundWork = ulFprs % (ulDivideWork * Freq);
			if(ulDivideWork == 1) {
				if((ulRoundWork * 2) >= (ulDivideWork * Freq)) {
					ulTDR00 += 1;
				}
			}
			else {
				if(ulRoundWork >= ((ulDivideWork / 2) * Freq)) {
					ulTDR00 += 1;
				}
			}
			TDR00H = (UCHAR)(ulTDR00 >> 8);
			TDR00L = (UCHAR)ulTDR00;
			break;
		}
	}
	TO0  &= (UCHAR)~0x01;	/* TO00  = 0 */
	TOE0 &= (UCHAR)~0x01;	/* TOE00 = 0 */
}

/*******************************************************************************
* Function Name: PWM_CH2_Init
* Description  : initializing the PWM output that uses the Timer Array Unit(Cannel2).
* Arguments    : Freq    ... setting frequency
* Return Value : none
********************************************************************************/
void PWM_CH2_Init( ULONG Freq )
{
#if defined DEVICE_R5F10Y4xASP

#if ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH2_USED) )
	/* Enables timer array unit input clock supply(TAU0EN=1) */
	PER0 |= 0x01;

	TT0  |= 0x05;			/* TT02  = 1 */
	
	PWM_CH0_Init(Freq);

	TMR02H = 0x04;
	TMR02L = 0x09;
	TOM0 |= 0x04;			/* TOM02 = 1 */
	TOL0 &= (UCHAR)~0x04;	/* TOL02 = 0 */
	TO0  &= (UCHAR)~0x04;	/* TO02  = 0 */
	TOE0 &= (UCHAR)~0x04;	/* TOE02 = 0 */

	IF0L &= (UCHAR)~0x80;	/* TMIF00 = 0 */
	MK0L &= (UCHAR)~0x80;	/* TMMK00 = 0 */
	TS0  |= 0x01;			/* TS00   = 1(timer start) */
#endif /* end of ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH2_USED) ) */

#endif /* end of defined DEVICE_R5F10Y4xASP */
}

/*******************************************************************************
* Function Name: PWM_CH3_Init
* Description  : initializing the PWM output that uses the Timer Array Unit(Cannel3).
* Arguments    : Freq    ... setting frequency
* Return Value : none
********************************************************************************/
void PWM_CH3_Init( ULONG Freq )
{
#if defined DEVICE_R5F10Y4xASP

#if ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH3_USED) )
	/* Enables timer array unit input clock supply(TAU0EN=1) */
	PER0 |= 0x01;

	TT0  |= 0x09;			/* TT03  = 1 */
	
	PWM_CH0_Init(Freq);

	TMR03H = 0x04;
	TMR03L = 0x09;
	TOM0 |= 0x08;			/* TOM03 = 1 */
	TOL0 &= (UCHAR)~0x08;	/* TOL03 = 0 */
	TO0  &= (UCHAR)~0x08;	/* TO03  = 0 */
	TOE0 &= (UCHAR)~0x08;	/* TOE03 = 0 */

	IF0L &= (UCHAR)~0x80;	/* TMIF00 = 0 */
	MK0L &= (UCHAR)~0x80;	/* TMMK00 = 0 */
	TS0  |= 0x01;			/* TS00   = 1(timer start) */
#endif /* end of ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH3_USED) ) */

#endif /* end of defined DEVICE_R5F10Y4xASP */
}

/*******************************************************************************
* Function Name: PWM_Out_CH2
* Description  : PWM that uses the Timer Array Unit(Cannel2) output.
* Arguments    : none
* Return Value : none
********************************************************************************/
void PWM_Out_CH2( void )
{
#if defined DEVICE_R5F10Y4xASP

#if ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH2_USED) )
	UCHAR	ucreg;
	UCHAR	regTDR02L;

	if (tblgPWM[ucgPWM_Cnt].Input == HIGH){
		if (tblgPWM[ucgPWM_Cnt].SetDuty != tblgPWM[ucgPWM_Cnt].OutDuty){
			switch (tblgPWM[ucgPWM_Cnt].SetDuty){
				case 0:
					TT0  |= 0x04;			/* TT02  = 1(timer stop) */
					TOE0 &= (UCHAR)~0x04;	/* TOE02 = 0 */
					PORT_PWM_CH2 = LOW;
					break;
				case 100:
					TT0  |= 0x04;			/* TT02  = 1(timer stop) */
					TOE0 &= (UCHAR)~0x04;	/* TOE02 = 0 */
					PORT_PWM_CH2 = HIGH;
					break;
				default:
					ucreg  = TDR00L;
					regTDR02L = PWM_Calc_Duty(ucreg);
					TT0  |= 0x04;			/* TT02  = 1(timer stop) */
					TDR02H = 0x00;
					TDR02L = regTDR02L;
					PORT_PWM_CH2 = LOW;
					TOE0 |= 0x04;			/* TOE02 = 1 */
					TS0  |= 0x04;			/* TS02  = 1(timer start) */
					break;
			}
		}

	}else if (tblgPWM[ucgPWM_Cnt].Input == LOW){
		tblgPWM[ucgPWM_Cnt].SetDuty = 0;
		TT0  |= 0x04;						/* TT02  = 1(timer stop) */
		TOE0 &= (UCHAR)~0x04;				/* TOE02 = 0 */
		PORT_PWM_CH2 = LOW;
	}
	tblgPWM[ucgPWM_Cnt].OutDuty = tblgPWM[ucgPWM_Cnt].SetDuty;
#endif /* end of ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH2_USED) ) */

#endif /* end of defined DEVICE_R5F10Y4xASP */
}

/*******************************************************************************
* Function Name: PWM_Out_CH3
* Description  : PWM that uses the Timer Array Unit(Cannel2) output.
* Arguments    : none
* Return Value : none
********************************************************************************/
void PWM_Out_CH3( void )
{
#if defined DEVICE_R5F10Y4xASP

#if ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH3_USED) )
	UCHAR	ucreg;
	UCHAR	regTDR03L;

	if (tblgPWM[ucgPWM_Cnt].Input == HIGH){
		if (tblgPWM[ucgPWM_Cnt].SetDuty != tblgPWM[ucgPWM_Cnt].OutDuty){
			switch (tblgPWM[ucgPWM_Cnt].SetDuty){
				case 0:
					TT0  |= 0x08;			/* TT03  = 1(timer stop) */
					TOE0 &= (UCHAR)~0x08;	/* TOE03 = 0 */
					PORT_PWM_CH3 = LOW;
					break;
				case 100:
					TT0  |= 0x08;			/* TT03  = 1(timer stop) */
					TOE0 &= (UCHAR)~0x08;	/* TOE03 = 0 */
					PORT_PWM_CH3 = HIGH;
					break;
				default:
					ucreg  = TDR00L;
					regTDR03L = PWM_Calc_Duty(ucreg);
					TT0  |= 0x08;			/* TT03  = 1(timer stop) */
					TDR03H = 0x00;
					TDR03L = regTDR03L;
					PORT_PWM_CH3 = LOW;
					TOE0 |= 0x08;			/* TOE03 = 1 */
					TS0  |= 0x08;			/* TS03  = 1(timer start) */
					break;
			}
		}

	}else if (tblgPWM[ucgPWM_Cnt].Input == LOW){
		tblgPWM[ucgPWM_Cnt].SetDuty = 0;
		TT0  |= 0x08;						/* TT03  = 1(timer stop) */
		TOE0 &= (UCHAR)~0x08;				/* TOE03 = 0 */
		PORT_PWM_CH3 = LOW;
	}
	tblgPWM[ucgPWM_Cnt].OutDuty = tblgPWM[ucgPWM_Cnt].SetDuty;
#endif /* end of ( !defined(_IARCOMPILER) && !defined(__GCC__) ) || defined(_IARCOMPILER) || ( defined(__GCC__) && defined(PWM_CH3_USED) ) */

#endif /* end of defined DEVICE_R5F10Y4xASP */
}

#endif /* end of defined PWM_USED */
